#!/usr/bin/env python3
"""
not1mm Contest logger
Email: michael.bridak@gmail.com
GPL V3
Class: BandMapWindow
Purpose: Onscreen widget to show realtime spots from an AR cluster.
"""

# pylint: disable=unused-import, c-extension-no-member, no-member, invalid-name, too-many-lines
# pylint: disable=logging-fstring-interpolation, line-too-long, no-name-in-module

import logging
import os
import platform
from json import loads

from PyQt6.QtWidgets import QDockWidget

import not1mm.fsutils as fsutils
from not1mm.lib.multicast import Multicast
from not1mm.lib.lookup import QRZlookup, HamQTH

logger = logging.getLogger(__name__)


class LookupService(QDockWidget):
    """The Lookup Service class."""

    multicast_interface = None

    def __init__(self):
        super().__init__()
        self._udpwatch = None

        self.settings = self.get_settings()
        self.look_up = None
        if self.settings.get("useqrz"):
            self.look_up = QRZlookup(
                self.settings.get("lookupusername"),
                self.settings.get("lookuppassword"),
            )

        if self.settings.get("usehamqth"):
            self.look_up = HamQTH(
                self.settings.get("lookupusername"),
                self.settings.get("lookuppassword"),
            )

        self.multicast_interface = Multicast(
            self.settings.get("multicast_group", "239.1.1.1"),
            self.settings.get("multicast_port", 2239),
            self.settings.get("interface_ip", "0.0.0.0"),
        )
        self.multicast_interface.ready_read_connect(self.watch_udp)

    def get_settings(self) -> dict:
        """Get the settings."""
        if os.path.exists(fsutils.CONFIG_FILE):
            with open(fsutils.CONFIG_FILE, "rt", encoding="utf-8") as file_descriptor:
                return loads(file_descriptor.read())

    def watch_udp(self):
        """doc"""
        while self.multicast_interface.server_udp.hasPendingDatagrams():
            packet = self.multicast_interface.read_datagram_as_json()

            if packet.get("station", "") != platform.node():
                continue
            if packet.get("cmd", "") == "LOOKUP_CALL":
                if self.look_up:
                    call = packet.get("call", "")
                    if call:
                        result = self.look_up.lookup(call)
                        cmd = {}
                        cmd["cmd"] = "LOOKUP_RESPONSE"
                        cmd["station"] = platform.node()
                        cmd["result"] = result
                        self.multicast_interface.send_as_json(cmd)
                continue

            if packet.get("cmd", "") == "REFRESH_LOOKUP":
                self.settings = self.get_settings()
                self.look_up = None
                if self.settings.get("useqrz"):
                    self.look_up = QRZlookup(
                        self.settings.get("lookupusername"),
                        self.settings.get("lookuppassword"),
                    )

                if self.settings.get("usehamqth"):
                    self.look_up = HamQTH(
                        self.settings.get("lookupusername"),
                        self.settings.get("lookuppassword"),
                    )



#!/usr/bin/env python3

"""
fsutils.py: Filesystem utilities for not1mm.
@kyleboyle
"""
# pylint: disable=invalid-name

import os
import platform
import sys
import subprocess
from pathlib import Path
from appdata import AppDataPaths

WORKING_PATH = Path(os.path.dirname(os.path.abspath(__file__)))
MODULE_PATH = WORKING_PATH
APP_DATA_PATH = MODULE_PATH / "data"
DATA_PATH = os.environ.get("XDG_DATA_HOME", str(Path.home() / ".local" / "share"))
DATA_PATH += "/not1mm"
USER_DATA_PATH = Path(DATA_PATH)
_CONFIG_PATH = os.environ.get("XDG_CONFIG_HOME", str(Path.home() / ".config"))
_CONFIG_PATH += "/not1mm"
CONFIG_PATH = Path(_CONFIG_PATH)
CONFIG_FILE = CONFIG_PATH / "not1mm.json"
LOG_FILE = USER_DATA_PATH / "not1mm_debug.log"

# Create directories if they do not exist on Linux systems
if platform.system() not in ["Windows", "Darwin"]:
    try:
        os.mkdir(CONFIG_PATH)
    except FileExistsError:
        ...
    try:
        os.mkdir(USER_DATA_PATH)
    except FileExistsError:
        ...

# Define and create directories if they do not exist on Windows or Mac systems
if platform.system() in ["Windows", "Darwin"]:
    _app_paths = AppDataPaths(name="not1mm")
    _app_paths.setup()
    LOG_FILE = _app_paths.get_log_file_path(name="appplication.log")
    _DATA_PATH = Path(_app_paths.app_data_path)
    USER_DATA_PATH = _DATA_PATH
    CONFIG_PATH = USER_DATA_PATH
    CONFIG_FILE = CONFIG_PATH / "not1mm.json"


def openFileWithOS(file):
    """Open a file with the default program for that OS."""
    if sys.platform == "win32":
        os.startfile(file)
    elif sys.platform == "darwin":
        subprocess.Popen(["open", file])
    else:
        subprocess.Popen(["xdg-open", file])



"""Edit Settings Dialog"""

from json import loads

from PyQt6 import QtWidgets, uic

from not1mm.lib.ham_utility import gridtolatlon


class EditStation(QtWidgets.QDialog):
    """Edit Station Settings"""

    cty_file = {}

    def __init__(self, app_data_path):
        super().__init__(None)
        uic.loadUi(app_data_path / "settings.ui", self)
        self.buttonBox.clicked.connect(self.store)
        self.GridSquare.textEdited.connect(self.gridchanged)
        self.Call.textEdited.connect(self.call_changed)
        with open(
            app_data_path / "cty.json", "rt", encoding="utf-8"
        ) as file_descriptor:
            self.cty_file = loads(file_descriptor.read())

    def store(self):
        """dialog magic"""

    def gridchanged(self):
        """Populated the Lat and Lon fields when the gridsquare changes"""
        lat, lon = gridtolatlon(self.GridSquare.text())
        self.Latitude.setText(str(round(lat, 4)))
        self.Longitude.setText(str(round(lon, 4)))

    def call_changed(self):
        """Populate zones"""
        results = self.cty_lookup()
        if results:
            for result in results.items():
                self.CQZone.setText(str(result[1].get("cq", "")))
                self.ITUZone.setText(str(result[1].get("itu", "")))
                self.Country.setText(str(result[1].get("entity", "")))

    def cty_lookup(self):
        """Lookup callsign in cty.dat file"""
        callsign = self.Call.text()
        callsign = callsign.upper()
        for count in reversed(range(len(callsign))):
            searchitem = callsign[: count + 1]
            result = {
                key: val for key, val in self.cty_file.items() if key == searchitem
            }
            if not result:
                continue
            if result.get(searchitem).get("exact_match"):
                if searchitem == callsign:
                    return result
                continue
            return result
